searchNodes=[{"doc":"Documentation for MaCrud .","ref":"MaCrud.html","title":"MaCrud","type":"module"},{"doc":"Generates CRUD functions to DRY Phoenix Contexts. Assumes Ecto.Repo is being used as the repository. Uses the Ecto Schema source name to generate the pluralized name for the functions, and the module name to generate the singular name. This follows the same pattern as the Mix.Tasks.Phx.Gen.Context , so it should be straightforward to replace Phoenix's auto-generated functions with MaCrud. Usage To generate CRUD functions for a given schema, simply do defmodule MyApp.MyContext do alias MyApp.Repo alias MyApp.MySchema require MaCrud.Context MaCrud.Context . generate_functions MySchema end And the context will have all these functions available: defmodule MyApp.MyContext do alias MyApp.Repo alias MyApp.MySchema require MaCrud.Context ## Exists functions def my_schema_exists? ( id ) do import Ecto.Query , only : [ from : 2 ] query = from ( x in MySchema , where : x . id == ^ id ) Repo . exists? ( query ) end ## Get functions def get_my_schema ( id , opts \\\\ [ ] ) do assocs = opts [ :assoc ] || [ ] MySchema |&gt; Repo . get ( id ) |&gt; Repo . preload ( assocs ) end def get_my_schema! ( id , opts \\\\ [ ] ) do assocs = opts [ :assoc ] || [ ] MySchema |&gt; Repo . get! ( id ) |&gt; Repo . preload ( assocs ) end def get_my_schema_by ( clauses , opts \\\\ [ ] ) do assocs = opts [ :assoc ] || [ ] MySchema |&gt; Repo . get_by ( clauses ) |&gt; Repo . preload ( assocs ) end def get_my_schema_by! ( clauses , opts \\\\ [ ] ) do assocs = opts [ :assoc ] || [ ] MySchema |&gt; Repo . get_by! ( clauses ) |&gt; Repo . preload ( assocs ) end ## List functions def list_my_schemas ( ) do Repo . all ( MySchema ) end def list_my_schemas ( opts ) do MySchema |&gt; MaCrud.Query . list ( opts ) |&gt; Repo . all ( ) end def list_my_schemas_with_assocs ( assocs ) do MySchema |&gt; Repo . all ( ) |&gt; Repo . preload ( assocs ) end def list_my_schemas_with_assocs ( assocs , opts ) do MySchema |&gt; MaCrud.Query . list ( opts ) |&gt; Repo . all ( ) |&gt; Repo . preload ( assocs ) end def filter_my_schemas ( filters ) do MySchema |&gt; MaCrud.Query . filter ( filters ) |&gt; Repo . all ( ) end def search_my_schemas ( search_term ) do module_fields = MySchema . __schema__ ( :fields ) MySchema |&gt; MaCrud.Query . search ( search_term , module_fields ) |&gt; Repo . all ( ) end def count_my_schemas ( field \\\\ :id ) do Repo . aggregate ( MySchema , :count , field ) end ## Create functions def create_my_schema ( attrs ) do % MySchema { } |&gt; MySchema . changeset ( attrs ) |&gt; Repo . insert ( ) end def create_my_schema! ( attrs ) do % MySchema { } |&gt; MySchema . changeset ( attrs ) |&gt; Repo . insert! ( ) end ## Update functions def update_my_schema ( % MySchema { } = my_schema , attrs ) do my_schema |&gt; MySchema . changeset ( attrs ) |&gt; Repo . update ( ) end def update_my_schema! ( % MySchema { } = my_schema , attrs ) do my_schema |&gt; MySchema . changeset ( attrs ) |&gt; Repo . update! ( ) end def update_my_schema_with_assocs ( % MySchema { } = my_schema , attrs , assocs ) do my_schema |&gt; Repo . preload ( assocs ) |&gt; MySchema . changeset ( attrs ) |&gt; Repo . update ( ) end def update_my_schema_with_assocs! ( % MySchema { } = my_schema , attrs , assocs ) do my_schema |&gt; Repo . preload ( assocs ) |&gt; MySchema . changeset ( attrs ) |&gt; Repo . update! ( ) end ## Delete functions def delete_my_schema ( % MySchema { } = my_schema ) do my_schema |&gt; Ecto.Changeset . change ( ) |&gt; check_assocs ( [ ] ) |&gt; Repo . delete ( ) end def delete_my_schema! ( % MySchema { } = my_schema ) do my_schema |&gt; Ecto.Changeset . change ( ) |&gt; check_assocs ( [ ] ) |&gt; Repo . delete! ( ) end def change_my_schema ( % MySchema { } = my_schema , attrs \\ %{ } ) do my_schema |&gt; MySchema . changeset ( attrs ) end # Function to check no_assoc_constraints, always generated. defp check_assocs ( changeset , nil ) , do : changeset defp check_assocs ( changeset , constraints ) do Enum . reduce ( constraints , changeset , fn i , acc -&gt; Ecto.Changeset . no_assoc_constraint ( acc , i ) end ) end end","ref":"MaCrud.Context.html","title":"MaCrud.Context","type":"module"},{"doc":"Sets default options for the context. Options :create - the name of the changeset function used in the create function. Defaults to :changeset . :update - the name of the changeset function used in the update function. Defaults to :changeset . :only - list of functions to be generated. If not empty, functions not specified in this list are not generated. Defaults to [] . :except - list of functions to not be generated. If not empty, only functions not specified in this list will be generated. Defaults to [] . The accepted values for :only and :except are: [:exists, :get, :list, :count, :search, :filter, :create, :update, :delete, :change] . Examples iex&gt; MaCrud.Context . default create : :create_changeset , update : :update_changeset :ok iex&gt; MaCrud.Context . default only : [ :create , :list ] :ok iex&gt; MaCrud.Context . default except : [ :get! , :list , :delete ] :ok","ref":"MaCrud.Context.html#default/1","title":"MaCrud.Context.default/1","type":"macro"},{"doc":"Generates CRUD functions for the schema_module . Custom options can be given. To see the available options, refer to the documenation of MaCrud.Context.default/1 . There is also one extra option that cannot be set by default: check_constraints_on_delete - list of associations that must be empty to allow deletion. Ecto.Changeset.no_assoc_constraint will be called for each association before deleting. Defaults to [] . Examples Suppose we want to implement basic CRUD functionality for a User schema, exposed through an Accounts context: defmodule MyApp.Accounts do alias MyApp.Repo require MaCrud.Context # Assuming Accounts.User implements a `changeset/2` function, used both to create and update a user. MaCrud.Context . generate_functions Accounts.User end Now, suppose the changeset for create and update are different, and we want to delete the record only if the association has_many :assocs is empty: defmodule MyApp.MyContext do alias MyApp.Repo alias MyApp.MySchema require MaCrud.Context MaCrud.Context . generate_functions MySchema , create : :create_changeset , update : :update_changeset , check_constraints_on_delete : [ :assocs ] end","ref":"MaCrud.Context.html#generate_functions/2","title":"MaCrud.Context.generate_functions/2","type":"macro"},{"doc":"Generates Ecto Queries. All functions in this module return an Ecto.Query . Combining the functions in this module can be very powerful. For example, to do pagination with filter and search: pagination_params = %{ limit : 10 , offset : 1 , order_by : &quot;id&quot; , sorting_order : :desc } filter_params = %{ username : [ &quot;username1&quot; , &quot;username2&quot; ] } search_params = %{ text : &quot;search text&quot; , fields : [ :username ] } User |&gt; MaCrud.Query . filter ( filter_params ) |&gt; MaCrud.Query . list ( pagination_params ) |&gt; MaCrud.Query . search ( search_params . text , search_params . fields ) |&gt; Repo . all ( )","ref":"MaCrud.Query.html","title":"MaCrud.Query","type":"module"},{"doc":"Filters the query. Examples MaCrud.Query . filter ( MySchema , %{ id : 5 , name : &quot;John&quot; } ) MaCrud.Query . filter ( MySchema , %{ name : [ &quot;John&quot; , &quot;Doe&quot; ] } )","ref":"MaCrud.Query.html#filter/2","title":"MaCrud.Query.filter/2","type":"function"},{"doc":"Applies some restrictions to the query. Expects opts to be a keyword list or a map containing some of these fields: limit : defaults to not limiting offset : defaults to 0 sorting_order : defaults to :asc (only works if there is also a order_by specified) order_by : defaults to not ordering custom_query : A function that receives the initial query as argument and returns a custom query. Defaults to initial_query Examples MaCrud.Query . list ( MySchema , [ limit : 10 ] ) MaCrud.Query . list ( MySchema , [ limit : 10 , offset : 3 , sorting_order : :desc , order_by : :value ] ) MaCrud.Query . list ( MySchema , %{ order_by : &quot;value&quot; } ) MaCrud.Query . list ( MySchema , %{ order_by : :value } ) MaCrud.Query . list ( MySchema , %{ order_by : [ &quot;age&quot; , &quot;username&quot; ] } ) MaCrud.Query . list ( MySchema , %{ order_by : [ :age , :username ] } ) MaCrud.Query . list ( MySchema , %{ order_by : [ asc : :age , desc : :username ] } ) MaCrud.Query . list ( MySchema , %{ order_by : [ %{ field : &quot;age&quot; , order : :asc } , %{ field : &quot;username&quot; , order : :desc } ] } ) MaCrud.Query . list ( MySchema , custom_query : &amp; MySchema . scope_list / 1 )","ref":"MaCrud.Query.html#list/2","title":"MaCrud.Query.list/2","type":"function"},{"doc":"Searches for the search_term in the given fields . Examples MaCrud.Query . search ( MySchema , &quot;John&quot; , [ :name ] )","ref":"MaCrud.Query.html#search/3","title":"MaCrud.Query.search/3","type":"function"},{"doc":"A module providing Internationalization with a gettext-based API. By using Gettext , your module gains a set of macros for translations, for example: import MaCrud.Gettext # Simple translation gettext ( &quot;Here is the string to translate&quot; ) # Plural translation ngettext ( &quot;Here is the string to translate&quot; , &quot;Here are the strings to translate&quot; , 3 ) # Domain-based translation dgettext ( &quot;errors&quot; , &quot;Here is the error message to translate&quot; ) See the Gettext Docs for detailed usage.","ref":"MaCrud.Translator.html","title":"MaCrud.Translator","type":"module"},{"doc":"Callback implementation for Gettext.Backend.dgettext/3 .","ref":"MaCrud.Translator.html#dgettext/3","title":"MaCrud.Translator.dgettext/3","type":"macro"},{"doc":"Callback implementation for Gettext.Backend.dgettext_noop/2 .","ref":"MaCrud.Translator.html#dgettext_noop/2","title":"MaCrud.Translator.dgettext_noop/2","type":"macro"},{"doc":"Callback implementation for Gettext.Backend.dngettext/5 .","ref":"MaCrud.Translator.html#dngettext/5","title":"MaCrud.Translator.dngettext/5","type":"macro"},{"doc":"Callback implementation for Gettext.Backend.dngettext_noop/3 .","ref":"MaCrud.Translator.html#dngettext_noop/3","title":"MaCrud.Translator.dngettext_noop/3","type":"macro"},{"doc":"Callback implementation for Gettext.Backend.dpgettext/4 .","ref":"MaCrud.Translator.html#dpgettext/4","title":"MaCrud.Translator.dpgettext/4","type":"macro"},{"doc":"","ref":"MaCrud.Translator.html#dpgettext_noop/3","title":"MaCrud.Translator.dpgettext_noop/3","type":"macro"},{"doc":"Callback implementation for Gettext.Backend.dpngettext/6 .","ref":"MaCrud.Translator.html#dpngettext/6","title":"MaCrud.Translator.dpngettext/6","type":"macro"},{"doc":"","ref":"MaCrud.Translator.html#dpngettext_noop/4","title":"MaCrud.Translator.dpngettext_noop/4","type":"macro"},{"doc":"","ref":"MaCrud.Translator.html#errors_domain/0","title":"MaCrud.Translator.errors_domain/0","type":"function"},{"doc":"Callback implementation for Gettext.Backend.gettext/2 .","ref":"MaCrud.Translator.html#gettext/2","title":"MaCrud.Translator.gettext/2","type":"macro"},{"doc":"Callback implementation for Gettext.Backend.gettext_comment/1 .","ref":"MaCrud.Translator.html#gettext_comment/1","title":"MaCrud.Translator.gettext_comment/1","type":"macro"},{"doc":"Callback implementation for Gettext.Backend.gettext_noop/1 .","ref":"MaCrud.Translator.html#gettext_noop/1","title":"MaCrud.Translator.gettext_noop/1","type":"macro"},{"doc":"Callback implementation for Gettext.Backend.handle_missing_bindings/2 .","ref":"MaCrud.Translator.html#handle_missing_bindings/2","title":"MaCrud.Translator.handle_missing_bindings/2","type":"function"},{"doc":"Callback implementation for Gettext.Backend.handle_missing_plural_translation/7 .","ref":"MaCrud.Translator.html#handle_missing_plural_translation/7","title":"MaCrud.Translator.handle_missing_plural_translation/7","type":"function"},{"doc":"Callback implementation for Gettext.Backend.handle_missing_translation/5 .","ref":"MaCrud.Translator.html#handle_missing_translation/5","title":"MaCrud.Translator.handle_missing_translation/5","type":"function"},{"doc":"","ref":"MaCrud.Translator.html#lgettext/5","title":"MaCrud.Translator.lgettext/5","type":"function"},{"doc":"","ref":"MaCrud.Translator.html#lngettext/7","title":"MaCrud.Translator.lngettext/7","type":"function"},{"doc":"Callback implementation for Gettext.Backend.ngettext/4 .","ref":"MaCrud.Translator.html#ngettext/4","title":"MaCrud.Translator.ngettext/4","type":"macro"},{"doc":"Callback implementation for Gettext.Backend.ngettext_noop/2 .","ref":"MaCrud.Translator.html#ngettext_noop/2","title":"MaCrud.Translator.ngettext_noop/2","type":"macro"},{"doc":"Callback implementation for Gettext.Backend.pgettext/3 .","ref":"MaCrud.Translator.html#pgettext/3","title":"MaCrud.Translator.pgettext/3","type":"macro"},{"doc":"","ref":"MaCrud.Translator.html#pgettext_noop/2","title":"MaCrud.Translator.pgettext_noop/2","type":"macro"},{"doc":"Callback implementation for Gettext.Backend.pngettext/5 .","ref":"MaCrud.Translator.html#pngettext/5","title":"MaCrud.Translator.pngettext/5","type":"macro"},{"doc":"","ref":"MaCrud.Translator.html#pngettext_noop/3","title":"MaCrud.Translator.pngettext_noop/3","type":"macro"},{"doc":"","ref":"MaCrud.Translator.html#schemas_domain/0","title":"MaCrud.Translator.schemas_domain/0","type":"function"}]